@@ -1,5 +1,5 @@
 package DBIx::Perlish;
-# $Id: Perlish.pm,v 1.80 2007/10/09 13:18:15 tobez Exp $
+# $Id: Perlish.pm,v 1.83 2007/10/16 12:32:47 tobez Exp $
 
 use 5.008;
 use warnings;
@@ -10,7 +10,7 @@ use vars qw($VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS $SQL @BIND_VALUES);
 require Exporter;
 use base 'Exporter';
 
-$VERSION = '0.29';
+$VERSION = '0.30';
 @EXPORT = qw(db_fetch db_select db_update db_delete db_insert sql);
 @EXPORT_OK = qw(union intersect except);
 %EXPORT_TAGS = (all => [@EXPORT, @EXPORT_OK]);
@@ -245,10 +245,6 @@ sub gen_sql
 	}
 	$sql .= join ", ", map { $tabs{$_} } sort keys %tabs;
 
-	$S->{sets}     ||= [];
-	$S->{where}    ||= [];
-	$S->{group_by} ||= [];
-	$S->{order_by} ||= [];
 	my @sets     = grep { $_ ne "" } @{$S->{sets}};
 	my @where    = grep { $_ ne "" } @{$S->{where}};
 	my @group_by = grep { $_ ne "" } @{$S->{group_by}};
@@ -272,9 +268,9 @@ sub gen_sql
 	if ($S->{offset}) {
 		$sql .= " offset $S->{offset}";
 	}
-	my $v = $S->{set_values} || [];
-	push @$v, @{$S->{ret_values} || []};
-	push @$v, @{$S->{values} || []};
+	my $v = $S->{set_values};
+	push @$v, @{$S->{ret_values}};
+	push @$v, @{$S->{values}};
 
 	for my $add (@{$S->{additions}}) {
 		$sql .= " $add->{type} $add->{sql}";
@@ -294,7 +290,7 @@ DBIx::Perlish - a perlish interface to SQL databases
 
 =head1 VERSION
 
-This document describes DBIx::Perlish version 0.29
+This document describes DBIx::Perlish version 0.30
 
 
 =head1 SYNOPSIS
@@ -1054,6 +1050,19 @@ will generate the equivalent to C<select * from products where type = 'ICBM'>,
 while the same code would generate just C<select * from products> if C<$type>
 were false.
 
+Similarly,
+
+    my $want_z = 1;
+    db_fetch {
+        my $p : products;
+        return $p->x, $p->y         unless $want_z;
+        return $p->x, $p->y, $p->z  if     $want_z;
+    };
+
+will generate the equivalent of C<select x, y from products> when
+C<$want_z> is false, and C<select x, y, z from products> when
+C<$want_z> is true.
+
 
 =head3 Statements with label syntax
 
@@ -1364,6 +1373,24 @@ cannot be done in Perl"?
 
 DBIx::Perlish requires no configuration files or environment variables.
 
+=head2 Running under L<Devel::Cover>
+
+When the C<DBIx::Perlish> module detects that the current program
+is being run under L<Devel::Cover>,
+it tries to cheat a little bit and feeds L<Devel::Cover>
+with I<false> information to make those
+query subs which were parsed by the module
+to appear "covered".
+
+This is done because the query subs are B<never> executed,
+and thus would normally be presented as "not covered" by
+the L<Devel::Cover> reporter.
+Although a developer has no trouble deciding to ignore
+such "red islands", he has to perform this decision every
+time he looks at the coverage data, which tends to become
+annoying rather quickly.
+
+Currently, only statement and sub execution data are faked.
 
 =head1 DEPENDENCIES
 
@@ -1440,9 +1467,6 @@ This work is in part sponsored by Telia Denmark.
 
 =head1 SUPPORT
 
-There is a project Wiki at
-  http://dbix-perlish.tobez.org/wiki/
-
 There is also the project website at
   http://dbix-perlish.tobez.org/
 
